/*  Sesame - Storage and Querying architecture for RDF and RDF Schema
 *  Copyright (C) 2001-2006 Aduna
 *
 *  Contact: 
 *  	Aduna
 *  	Prinses Julianaplein 14 b
 *  	3817 CS Amersfoort
 *  	The Netherlands
 *  	tel. +33 (0)33 465 99 87
 *  	fax. +33 (0)33 465 99 87
 *
 *  	http://aduna-software.com/
 *  	http://www.openrdf.org/
 *  
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License as published by the Free Software Foundation; either
 *  version 2.1 of the License, or (at your option) any later version.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

package org.openrdf.sesame.query.rql.model.iterators;

import org.openrdf.model.Resource;
import org.openrdf.model.Value;

import org.openrdf.sesame.query.QueryEvaluationException;
import org.openrdf.sesame.sail.ResourceIterator;
import org.openrdf.sesame.sail.ValueIterator;

public abstract class NestedValueIterator implements ValueIterator {

	protected ResourceIterator _outerIter;
	protected ValueIterator _innerIter;

	public NestedValueIterator(ResourceIterator outerIter) {
		_outerIter = outerIter;
	}

	public boolean hasNext() throws QueryEvaluationException {
		if (_innerIter == null || !_innerIter.hasNext()) {
			// Current inner iterator exhausted
            Resource outerResource;
			while (_outerIter.hasNext()) {
				outerResource = _outerIter.nextResource();
				_innerIter = getInnerIterator(outerResource);

				if (_innerIter.hasNext()) {
					break;
				}

				// Otherwise: get iterator for next outer value
			}
		}

		if (_innerIter != null && _innerIter.hasNext()) {
			return true;
		}

		return false;
	}

	public Value next() throws QueryEvaluationException {
		if (hasNext()) {
			return _innerIter.next();
		}
		throw new java.util.NoSuchElementException();
	}

	public abstract ValueIterator getInnerIterator(Resource outerResource);

	public void close() {
		if (_outerIter != null) {
			_outerIter.close();
			_outerIter = null;
		}
		if (_innerIter != null) {
			_innerIter.close();
			_innerIter = null;
		}
	}

	protected void finalize() {
		close();
	}
}
